﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Stimulsoft.Report;
using Stimulsoft.Report.Components;
using Stimulsoft.Base.Drawing;

namespace TestPrintStimulsoftWpf
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            PrintDialog pd = new PrintDialog();
            textBox1.Text = pd.PrintQueue.FullName;
        }

        private void buttonExit_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private StiReport GetPreparedReport()
        {
            StiReport report = new StiReport();
            report.Unit = Stimulsoft.Report.Units.StiMillimetersUnit.Millimeters;
            StiPage page = new StiPage();
            report.Pages.Clear();
            report.Pages.Add(page);
            page.PaperSize = System.Drawing.Printing.PaperKind.A4;
            page.Margins = new StiMargins(10, 10, 10, 10);
            page.Brush = new StiSolidBrush(System.Drawing.Color.White);

            if (checkBoxText.IsChecked.GetValueOrDefault(false))
            {
                StiText text = new StiText();
                text.ClientRectangle = new Stimulsoft.Base.Drawing.RectangleD(0, 0, 190, 50);
                text.Text = "Test string, page {PageNumber}";
                text.Font = new System.Drawing.Font("Arial", 20);
                text.Border = new Stimulsoft.Base.Drawing.StiBorder(Stimulsoft.Base.Drawing.StiBorderSides.All, System.Drawing.Color.Blue, 2, Stimulsoft.Base.Drawing.StiPenStyle.Solid);
                text.HorAlignment = StiTextHorAlignment.Center;
                text.VertAlignment = StiVertAlignment.Center;
                text.Brush = new StiSolidBrush(System.Drawing.Color.White);
                text.Report = report;
                page.Components.Add(text);
            }

            if (checkBoxImage.IsChecked.GetValueOrDefault(false))
            {
                StiImage image = new StiImage();
                image.ClientRectangle = new Stimulsoft.Base.Drawing.RectangleD(80, 80, 30, 30);
                image.ImageURL = new StiImageURLExpression("Logo.png");
                image.Stretch = true;
                image.AspectRatio = true;
                image.Border = new Stimulsoft.Base.Drawing.StiBorder(Stimulsoft.Base.Drawing.StiBorderSides.All, System.Drawing.Color.Red, 2, Stimulsoft.Base.Drawing.StiPenStyle.Solid);
                page.Components.Add(image);
            }

            report.RenderWithWpf();

            return report;
        }

        private void buttonShow_Click(object sender, RoutedEventArgs e)
        {
            var report = GetPreparedReport();
            report.ShowWithWpf(true);
        }

        private void buttonPrint_Click(object sender, RoutedEventArgs e)
        {
            var report = GetPreparedReport();

            bool showPrintDialog = checkBoxShowPrintDialog.IsChecked.GetValueOrDefault(false);

            StiWpfPrintProviderTest printProvider = new StiWpfPrintProviderTest(report);
            printProvider.Print(null, showPrintDialog, 1, 1, 1, textBox1.Text);
        }

    }
}
