﻿#region Copyright (C) 2003-2015 Stimulsoft
/*
{*******************************************************************}
{																	}
{	Stimulsoft Reports.Wpf											}
{	                         										}
{																	}
{	Copyright (C) 2003-2015 Stimulsoft     							}
{	ALL RIGHTS RESERVED												}
{																	}
{	The entire contents of this file is protected by U.S. and		}
{	International Copyright Laws. Unauthorized reproduction,		}
{	reverse-engineering, and distribution of all or any portion of	}
{	the code contained in this file is strictly prohibited and may	}
{	result in severe civil and criminal penalties and will be		}
{	prosecuted to the maximum extent possible under the law.		}
{																	}
{	RESTRICTIONS													}
{																	}
{	THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES			}
{	ARE CONFIDENTIAL AND PROPRIETARY								}
{	TRADE SECRETS OF Stimulsoft										}
{																	}
{	CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON		}
{	ADDITIONAL RESTRICTIONS.										}
{																	}
{*******************************************************************}
*/
#endregion Copyright (C) 2003-2015 Stimulsoft

using System;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using System.Collections.Generic;
using Stimulsoft.Base;
using Stimulsoft.Base.Drawing;
using Stimulsoft.Base.Localization;
using Stimulsoft.Report.Components;
using Stimulsoft.Report.Units;
using Stimulsoft.Report.Wpf;
using Stimulsoft.Report.Events;
using Stimulsoft.Report.Painters;

namespace TestPrintStimulsoftWpf
{
    public class StiPageWpfPainterTest : StiContainerWpfPainter
    {       
        #region Methods.Watermark
        public virtual void PaintText(StiWatermark watermark, DrawingContext dc, Rect rect, bool isPrinting)
        {
            if (watermark.Enabled && !string.IsNullOrEmpty(watermark.Text))
            {
                var textInfo = new StiTextInfo
                {
                    Text = watermark.Text,
                    Brush = StiBrushHelper.BrushToWpfBrush(watermark.TextBrush),
                    Rect = rect,
                    Font = watermark.Font,
                    HorizontalAlignment = TextAlignment.Center,
                    VerticalAlignment = VerticalAlignment.Center,
                    Angle = watermark.Angle,
                    TextTrimming = TextTrimming.None,
                    TextWrapping = TextWrapping.NoWrap,
                    FlowDirection = watermark.RightToLeft ? FlowDirection.RightToLeft : FlowDirection.LeftToRight
                };

                if (watermark.Font != null)
                    textInfo.FontSizeInPoints = watermark.Font.Size;

                StiTextHelper.DrawText(dc, textInfo);
            }
        }

        public void PaintWatermark(StiPage page, DrawingContext dc, bool isBehind)
        {
            double pgWidth = page.Unit.ConvertToHInches(page.DisplayRectangle.Width);
            double pgHeight = page.Unit.ConvertToHInches(page.DisplayRectangle.Height);

            Rect rect = new Rect(0, 0, pgWidth, pgHeight);

            //if (page.Watermark.ShowImageBehind == isBehind) PaintImage(page.Watermark, dc, rect);
            if (page.Watermark.ShowBehind == isBehind) PaintText(page.Watermark, dc, rect, page.IsPrinting);
        }
        #endregion

        #region Methods.Painter
        public override void Paint(StiComponent comp, StiPaintEventArgs e)
        {
            if (!(e.Context is DrawingContext))
                throw new Exception("StiWpfPainter can work only with System.Windows.Media.DrawingContext context!");

            var page = comp as StiPage;

            var dc = e.Context as DrawingContext;
            page.InvokePainting(page, e);

            StiUnit unit = page.Unit;

            double mgLeft = unit.ConvertToHInches(page.Margins.Left);
            double mgTop = unit.ConvertToHInches(page.Margins.Top);
            double mgRight = unit.ConvertToHInches(page.Margins.Right);
            double mgBottom = unit.ConvertToHInches(page.Margins.Bottom);

            double pgWidth = unit.ConvertToHInches(page.DisplayRectangle.Width);
            double pgHeight = unit.ConvertToHInches(page.DisplayRectangle.Height);

            double wd = unit.ConvertToHInches(page.Width);
            double ht = unit.ConvertToHInches(page.Height);

            #region Watermark Behind
            PaintWatermark(page, dc, true);
            #endregion                
                                    
            #region Paint copyright before
            StiPageContextPaintEventArgs ppe = new StiPageContextPaintEventArgs(
                dc,
                e.ClipRectangle.ToRectangle(),
                new System.Drawing.Rectangle((int)mgLeft, (int)mgTop, (int)wd, (int)ht),
                new System.Drawing.Rectangle(0, 0, (int)pgWidth, (int)pgHeight),
                page.Report.IsPrinting, page.Report.IsDesigning);

            dc.PushTransform(new TranslateTransform(-mgLeft, -mgTop));
            StiPage.InvokePagePainting(page, ppe);
            dc.Pop();
            #endregion

            e.Cancel = false;
            page.InvokePainted(page, e);

            dc.PushTransform(new TranslateTransform(mgLeft, mgTop));
            
            #region Paint Components
            base.PaintComponents(page, e);
            #endregion

            dc.Pop();

            #region Paint Border
            dc.PushTransform(new TranslateTransform(mgLeft, mgTop));
            PaintBorder(page, dc, new Rect(0, 0, wd, ht), true, true);
            dc.Pop();
            #endregion
            
            #region Paint Watermark at top
            dc.PushTransform(new TranslateTransform(-mgLeft, -mgTop));
            PaintWatermark(page, dc, false);
            dc.Pop();
            #endregion

            #region Paint copyright after
            dc.PushTransform(new TranslateTransform(-mgLeft, -mgTop));
            StiPage.InvokePagePainted(page, ppe);
            dc.Pop();
            #endregion            
        }
        #endregion
    }
}