﻿#region Copyright (C) 2003-2015 Stimulsoft
/*
{*******************************************************************}
{																	}
{	Stimulsoft Reports.Wpf											}
{	                         										}
{																	}
{	Copyright (C) 2003-2015 Stimulsoft     							}
{	ALL RIGHTS RESERVED												}
{																	}
{	The entire contents of this file is protected by U.S. and		}
{	International Copyright Laws. Unauthorized reproduction,		}
{	reverse-engineering, and distribution of all or any portion of	}
{	the code contained in this file is strictly prohibited and may	}
{	result in severe civil and criminal penalties and will be		}
{	prosecuted to the maximum extent possible under the law.		}
{																	}
{	RESTRICTIONS													}
{																	}
{	THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES			}
{	ARE CONFIDENTIAL AND PROPRIETARY								}
{	TRADE SECRETS OF Stimulsoft										}
{																	}
{	CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON		}
{	ADDITIONAL RESTRICTIONS.										}
{																	}
{*******************************************************************}
*/
#endregion Copyright (C) 2003-2015 Stimulsoft

using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Documents;
using System.Windows.Controls;
using System.Printing;
using System.Collections.Generic;
using System.Text;
using System.Drawing.Printing;
using Stimulsoft.Report;
using Stimulsoft.Report.Components;

namespace TestPrintStimulsoftWpf
{
    public class StiWpfPrintProviderTest
    {
        #region Range
        public class Range
        {
            public int fromPage;
            public int toPage;

            public Range(int fromPage, int toPage)
            {
                this.fromPage = fromPage;
                this.toPage = toPage;
            }
        }
        #endregion

        #region Fields
        private StiReport report = null;
        #endregion

        #region Methods
        /// <summary>
        /// Prints the rendered report. If the report is not rendered then its rendering starts.
        /// </summary>
        /// <param name="printTicketObject">Specifies information about how a document is printed.</param>
        /// <param name="showPrintDialog">Show the print dialog or no.</param>
        /// <param name="fromPage">Number of the first page to print. Starts from 1.</param>
        /// <param name="toPage">Number of the last page to print. Starts from 1.</param>
        /// <param name="copies">Number of copies to print.</param>
        /// <param name="printerName">A name of a printer that will be used for printing.</param>
        [System.Security.SecuritySafeCritical]
        public void Print(object printTicketObject, bool showPrintDialog, int fromPage, int toPage, int copies, string printerName)
        {
            var printTicket = printTicketObject as PrintTicket;

            var printDialog = new PrintDialog();
            printDialog.UserPageRangeEnabled = true;

            if (printTicket != null)
                printDialog.PrintTicket = printTicket;

            #region Check Range
            int totalRenderedPageCount = report.GetTotalRenderedPageCount();
            if (totalRenderedPageCount < 1) totalRenderedPageCount = 1;
            
            if (toPage == -1 || toPage == 0) toPage = totalRenderedPageCount;
            if (fromPage < 1) fromPage = 1;
            if (toPage < 1) toPage = 1;
            fromPage = Math.Min(fromPage, totalRenderedPageCount);
            toPage = Math.Min(toPage, totalRenderedPageCount);
            #endregion

            printDialog.MinPage = (uint)fromPage;
            printDialog.MaxPage = (uint)toPage;

            if (printDialog.PrintTicket.CopyCount <= 1)
            {
                if (copies > 0)
                    printDialog.PrintTicket.CopyCount = copies;
                else
                    printDialog.PrintTicket.CopyCount = report.PrinterSettings.Copies;
            }            

            #region Select Printer Name
            if ((!string.IsNullOrEmpty(report.PrinterSettings.PrinterName)) || (!string.IsNullOrEmpty(printerName)))
            {
                PrintServer server = null;
                string partServerName = null;
                string partPrinterName = null;
                if (!string.IsNullOrEmpty(report.PrinterSettings.PrinterName))
                {
                    partPrinterName = report.PrinterSettings.PrinterName;
                    if (report.PrinterSettings.PrinterName.StartsWith(@"\\") && !report.PrinterSettings.PrinterName.Contains(@"http://"))
                    {
                        int posPrinterName = report.PrinterSettings.PrinterName.IndexOf('\\', 2);
                        if (posPrinterName != -1)
                        {
                            partServerName = report.PrinterSettings.PrinterName.Substring(0, posPrinterName);
                            partPrinterName = report.PrinterSettings.PrinterName.Substring(posPrinterName + 1);
                            server = new PrintServer(partServerName);
                        }
                    }
                    if (server == null) server = new LocalPrintServer();
                    printDialog.PrintQueue = server.GetPrintQueue(partPrinterName);
                }
                else if (!string.IsNullOrEmpty(printerName))
                {
                    partPrinterName = printerName;
                    if (printerName.StartsWith(@"\\") && !printerName.Contains(@"http://"))
                    {
                        int posPrinterName = printerName.IndexOf('\\', 2);
                        if (posPrinterName != -1)
                        {
                            partServerName = printerName.Substring(0, posPrinterName);
                            partPrinterName = printerName.Substring(posPrinterName + 1);
                            server = new PrintServer(partServerName);
                        }
                    }
                    if (server == null) server = new LocalPrintServer();
                    printDialog.PrintQueue = server.GetPrintQueue(partPrinterName);
                }
            }
            #endregion

            printDialog.PageRange = new PageRange(fromPage, toPage);

            if (report.PrinterSettings.ShowDialog && showPrintDialog)
            {
                bool? value = printDialog.ShowDialog().GetValueOrDefault();

                if (!value.HasValue) report.PrinterSettings.PrintDialogResult = System.Windows.Forms.DialogResult.None;
                else 
                {
                    if ((bool)value) report.PrinterSettings.PrintDialogResult = System.Windows.Forms.DialogResult.OK;
                    else report.PrinterSettings.PrintDialogResult = System.Windows.Forms.DialogResult.Cancel;
                }
            }
            else report.PrinterSettings.PrintDialogResult = System.Windows.Forms.DialogResult.None;

            if (report.PrinterSettings.PrintDialogResult == System.Windows.Forms.DialogResult.OK ||
                report.PrinterSettings.PrintDialogResult == System.Windows.Forms.DialogResult.None)
            {
                fromPage = printDialog.PageRange.PageFrom;
                toPage = printDialog.PageRange.PageTo;

                try
                {
                    BeginPrint(fromPage, toPage);

                    var ranges = GetRanges(report, fromPage, toPage);

                    foreach (var range in ranges)
                    {
                        #region Create format string
                        string name = string.IsNullOrEmpty(report.ReportAlias) ? report.ReportName : report.ReportAlias;
                        if (ranges.Count > 1)
                        {
                            if (report.RenderedPages.Count > 99)
                                name = string.Format("{0} {1:D3}-{2:D3}", name, range.fromPage, range.toPage);
                            else if (report.RenderedPages.Count > 9)
                                name = string.Format("{0} {1:D2}-{2:D2}", name, range.fromPage, range.toPage);
                            else
                                name = string.Format("{0} {1}-{2}", name, range.fromPage, range.toPage);
                        }
                        #endregion

                        int x = 0;
                        int y = 0;
                        int pageIndex = report.GetPageIndex(range.fromPage, out x, out y);

                        double printableAreaWidth = 0;
                        double printableAreaHeight = 0;

                        SetPrintTicketFromPage(printDialog, report.RenderedPages[pageIndex], ref printableAreaWidth, ref printableAreaHeight);

                        var document = new StiWpfDocumentPaginatorTest(report, range.fromPage, range.toPage,
                            printableAreaWidth, printableAreaHeight);
                        printDialog.PrintDocument(document, name);
                    }
                }
                finally
                {
                    StopPrint();
                }

                GC.Collect();
                GC.WaitForPendingFinalizers();
                GC.Collect();
            }
            else
            {
                report.IsStopped = true;
                report.PrinterSettings.PrintDialogResult = System.Windows.Forms.DialogResult.None;
            }
        }

        private void BeginPrint(int fromPage, int toPage)
        {
            foreach (StiPage page in report.RenderedPages)
            {
                page.IsPrinted = false;
            }
            report.PageNumber = fromPage;
            report.CurrentPrintPage = 0;
        }

        private void StopPrint()
        {
        }

        #region SetPrintTicketFromPage
        [System.Security.SecuritySafeCritical]
        public static void SetPrintTicketFromPage(PrintDialog printDialog, StiPage page, ref double printableAreaWidth, ref double printableAreaHeight)
        {
            printDialog.PrintTicket.Collation = page.Report.PrinterSettings.Collate ? Collation.Collated : Collation.Uncollated;
            switch (page.Report.PrinterSettings.Duplex)
            {
                case Duplex.Simplex:
                    printDialog.PrintTicket.Duplexing = Duplexing.OneSided;
                    break;
                case Duplex.Horizontal:
                    printDialog.PrintTicket.Duplexing = Duplexing.TwoSidedLongEdge;
                    break;
                case Duplex.Vertical:
                    printDialog.PrintTicket.Duplexing = Duplexing.TwoSidedShortEdge;
                    break;
            }
            
            double pageWidth = page.Unit.ConvertToHInches(page.PageWidth);
            double pageHeight = page.Unit.ConvertToHInches(page.PageHeight);

            double originPageWidth = pageWidth;
            double originPageHeight = pageHeight;
            if (page.Orientation == StiPageOrientation.Landscape && printDialog.PrintTicket.PageOrientation == PageOrientation.Portrait)
            {
                originPageWidth = pageHeight;
                originPageHeight = pageWidth;
            }
            PageOrientation pageOrientation = page.Orientation == StiPageOrientation.Portrait ? PageOrientation.Portrait : PageOrientation.Landscape;

            PageMediaSizeName mediaSizeName = GetPageMediaSizeName(page.PaperSize);
            if (mediaSizeName == PageMediaSizeName.Unknown)
            {
                Size size = GetPrintableArea(printDialog, originPageWidth, originPageHeight, pageOrientation);
                printableAreaWidth = size.Width;
                printableAreaHeight = size.Height;
                if (page.Orientation == StiPageOrientation.Landscape && printDialog.PrintTicket.PageOrientation == PageOrientation.Portrait)
                {
                    printableAreaWidth = size.Height;
                    printableAreaHeight = size.Width;
                }
            }
            else
            {
                printDialog.PrintTicket.PageMediaSize = new PageMediaSize(mediaSizeName, originPageWidth, originPageHeight);
                printDialog.PrintTicket.PageOrientation = pageOrientation;
                printableAreaWidth = printDialog.PrintableAreaWidth;
                printableAreaHeight = printDialog.PrintableAreaHeight;
            }

        }
        #endregion

        #region GetPrintableArea
        [System.Security.SecuritySafeCritical()]
        private static Size GetPrintableArea(PrintDialog printDialog, double originalPageWidth, double originalPageHeight, PageOrientation pageOrientation)
        {
            int pageWidth = (int)Math.Round(originalPageWidth, 0);
            int pageHeight = (int)Math.Round(originalPageHeight, 0);
            double delta = double.MaxValue;
            PaperSize storedPS = null;
            PageMediaSizeName mediaSizeName = PageMediaSizeName.Unknown;

            PrinterSettings printerSettings = new PrinterSettings();
            if (printDialog != null && printDialog.PrintQueue != null && printerSettings.PrinterName != printDialog.PrintQueue.Name)
            {
                printerSettings.PrinterName = printDialog.PrintQueue.Name;
            }

            if (printerSettings.PaperSizes != null)
            {
                foreach (PaperSize ps in printerSettings.PaperSizes)
                {
                    if ((ps.Width == pageWidth && ps.Height == pageHeight) ||
                        (ps.Height == pageWidth && ps.Width == pageHeight))
                    {
                        mediaSizeName = GetPageMediaSizeName(ps.Kind);
                        if (printDialog != null && printDialog.PrintTicket != null)
                        {
                            printDialog.PrintTicket.PageMediaSize = new PageMediaSize(mediaSizeName, pageWidth, pageHeight);
                            printDialog.PrintTicket.PageOrientation = pageOrientation; 
                            if (pageOrientation == PageOrientation.Portrait)
                                return new Size(printDialog.PrintableAreaWidth, printDialog.PrintableAreaHeight);
                            else
                                return new Size(printDialog.PrintableAreaHeight, printDialog.PrintableAreaWidth);
                        }
                    }
                    double tempDelta = Math.Abs(ps.Width - pageWidth) + Math.Abs(ps.Height - pageHeight);
                    if (tempDelta < delta)
                    {
                        delta = tempDelta;
                        storedPS = ps;
                    }
                }
            }

            if (printDialog != null && printDialog.PrintTicket != null && storedPS != null)
            {
                if (StiOptions.Wpf.UseNewWpfPrintingMethod)
                {
                    printDialog.PrintTicket.PageMediaSize = new PageMediaSize(originalPageWidth, originalPageHeight);
                    printDialog.PrintTicket.PageOrientation = pageOrientation;
                    return new Size(pageWidth, pageHeight);
                }
                else
                {
                    mediaSizeName = GetPageMediaSizeName(storedPS.Kind);

                    printDialog.PrintTicket.PageMediaSize = new PageMediaSize(mediaSizeName, storedPS.Width, storedPS.Height);
                    printDialog.PrintTicket.PageOrientation = (storedPS.Width < storedPS.Height ? PageOrientation.Portrait : PageOrientation.Landscape);

                    double scaleW = printDialog.PrintableAreaWidth / storedPS.Width;
                    double scaleH = printDialog.PrintableAreaHeight / storedPS.Height;
                    printDialog.PrintTicket.PageMediaSize = new PageMediaSize(originalPageWidth, originalPageHeight);
                    printDialog.PrintTicket.PageOrientation = pageOrientation;
                    return new Size(pageWidth * scaleW, pageHeight * scaleH);
                }
            }

            printDialog.PrintTicket.PageMediaSize = new PageMediaSize(originalPageWidth, originalPageHeight);
            printDialog.PrintTicket.PageOrientation = pageOrientation;
            return new Size(pageWidth, pageHeight);
        }
        #endregion

        #region GetPageMediaSizeName
        [System.Security.SecuritySafeCritical()]
        private static System.Printing.PageMediaSizeName GetPageMediaSizeName(System.Drawing.Printing.PaperKind pageKind)
        {
            if (pageKind == System.Drawing.Printing.PaperKind.A2)return PageMediaSizeName.ISOA2;
            if (pageKind == System.Drawing.Printing.PaperKind.A3)return PageMediaSizeName.ISOA3;
            if (pageKind == System.Drawing.Printing.PaperKind.A3Extra)return PageMediaSizeName.ISOA3Extra;
            //if (pageKind == System.Drawing.Printing.PaperKind.A3ExtraTransverse)return PageMediaSizeName.ISOA3ex;
            if (pageKind == System.Drawing.Printing.PaperKind.A3Rotated)return PageMediaSizeName.ISOA3Rotated;
            //if (pageKind == System.Drawing.Printing.PaperKind.A3Transverse)return PageMediaSizeName.ISOA3tr;
            if (pageKind == System.Drawing.Printing.PaperKind.A4)return PageMediaSizeName.ISOA4;
            if (pageKind == System.Drawing.Printing.PaperKind.A4Extra)return PageMediaSizeName.ISOA4Extra;
            if (pageKind == System.Drawing.Printing.PaperKind.A4Plus) return PageMediaSizeName.OtherMetricA4Plus;
            if (pageKind == System.Drawing.Printing.PaperKind.A4Rotated)return PageMediaSizeName.ISOA4Rotated;
            //if (pageKind == System.Drawing.Printing.PaperKind.A4Small)return PageMediaSizeName.ISOA4s;
            //if (pageKind == System.Drawing.Printing.PaperKind.A4Transverse)return PageMediaSizeName.ISOA4t;
            if (pageKind == System.Drawing.Printing.PaperKind.A5)return PageMediaSizeName.ISOA5;
            if (pageKind == System.Drawing.Printing.PaperKind.A5Extra)return PageMediaSizeName.ISOA5Extra;
            if (pageKind == System.Drawing.Printing.PaperKind.A5Rotated)return PageMediaSizeName.ISOA5Rotated;
            //if (pageKind == System.Drawing.Printing.PaperKind.A5Transverse)return PageMediaSizeName.ISOA5t;
            if (pageKind == System.Drawing.Printing.PaperKind.A6)return PageMediaSizeName.ISOA6;
            if (pageKind == System.Drawing.Printing.PaperKind.A6Rotated)return PageMediaSizeName.ISOA6Rotated;
            //if (pageKind == System.Drawing.Printing.PaperKind.APlus)return PageMediaSizeName.ISOA;
            if (pageKind == System.Drawing.Printing.PaperKind.B4)return PageMediaSizeName.ISOB4;
            if (pageKind == System.Drawing.Printing.PaperKind.B4Envelope)return PageMediaSizeName.ISOB4Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.B4JisRotated)return PageMediaSizeName.JISB4Rotated;
            //if (pageKind == System.Drawing.Printing.PaperKind.B5)return PageMediaSizeName.ISOB5;
            if (pageKind == System.Drawing.Printing.PaperKind.B5Envelope)return PageMediaSizeName.ISOB5Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.B5Extra)return PageMediaSizeName.ISOB5Extra;
            if (pageKind == System.Drawing.Printing.PaperKind.B5JisRotated)return PageMediaSizeName.JISB5Rotated;
            //if (pageKind == System.Drawing.Printing.PaperKind.B5Transverse)return PageMediaSizeName.ISOb5tr;
            //if (pageKind == System.Drawing.Printing.PaperKind.B6Envelope)return PageMediaSizeName.ISOb6;
            if (pageKind == System.Drawing.Printing.PaperKind.B6Jis)return PageMediaSizeName.JISB6;
            if (pageKind == System.Drawing.Printing.PaperKind.B6JisRotated)return PageMediaSizeName.JISB6Rotated;
            //if (pageKind == System.Drawing.Printing.PaperKind.BPlus)return PageMediaSizeName.bplus;
            if (pageKind == System.Drawing.Printing.PaperKind.C3Envelope)return PageMediaSizeName.ISOC3Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.C4Envelope)return PageMediaSizeName.ISOC4Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.C5Envelope)return PageMediaSizeName.ISOC5Envelope;
            //if (pageKind == System.Drawing.Printing.PaperKind.C65Envelope)return PageMediaSizeName.ISOc65;
            if (pageKind == System.Drawing.Printing.PaperKind.C6Envelope)return PageMediaSizeName.ISOC6Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.CSheet)return PageMediaSizeName.NorthAmericaArchitectureCSheet;
            if (pageKind == System.Drawing.Printing.PaperKind.Custom)return PageMediaSizeName.Unknown;
            if (pageKind == System.Drawing.Printing.PaperKind.DLEnvelope)return PageMediaSizeName.ISODLEnvelope;
            if (pageKind == System.Drawing.Printing.PaperKind.DSheet) return PageMediaSizeName.NorthAmericaArchitectureDSheet;
            if (pageKind == System.Drawing.Printing.PaperKind.ESheet) return PageMediaSizeName.NorthAmericaArchitectureESheet;
            if (pageKind == System.Drawing.Printing.PaperKind.Executive)return PageMediaSizeName.NorthAmericaExecutive;
            if (pageKind == System.Drawing.Printing.PaperKind.Folio) return PageMediaSizeName.OtherMetricFolio;
            if (pageKind == System.Drawing.Printing.PaperKind.GermanLegalFanfold)return PageMediaSizeName.NorthAmericaGermanLegalFanfold;
            if (pageKind == System.Drawing.Printing.PaperKind.GermanStandardFanfold)return PageMediaSizeName.NorthAmericaGermanStandardFanfold;
            if (pageKind == System.Drawing.Printing.PaperKind.InviteEnvelope) return PageMediaSizeName.OtherMetricInviteEnvelope;
            if (pageKind == System.Drawing.Printing.PaperKind.IsoB4)return PageMediaSizeName.ISOB4;
            if (pageKind == System.Drawing.Printing.PaperKind.ItalyEnvelope) return PageMediaSizeName.OtherMetricItalianEnvelope;
            if (pageKind == System.Drawing.Printing.PaperKind.JapaneseDoublePostcard)return PageMediaSizeName.JapanDoubleHagakiPostcard;
            if (pageKind == System.Drawing.Printing.PaperKind.JapaneseDoublePostcardRotated)return PageMediaSizeName.JapanDoubleHagakiPostcardRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.JapaneseEnvelopeChouNumber3)return PageMediaSizeName.JapanChou3Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.JapaneseEnvelopeChouNumber3Rotated)return PageMediaSizeName.JapanChou3EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.JapaneseEnvelopeChouNumber4)return PageMediaSizeName.JapanChou4Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.JapaneseEnvelopeChouNumber4Rotated)return PageMediaSizeName.JapanChou4EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.JapaneseEnvelopeKakuNumber2)return PageMediaSizeName.JapanKaku2Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.JapaneseEnvelopeKakuNumber2Rotated)return PageMediaSizeName.JapanKaku2EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.JapaneseEnvelopeKakuNumber3)return PageMediaSizeName.JapanKaku3Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.JapaneseEnvelopeKakuNumber3Rotated)return PageMediaSizeName.JapanKaku3EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.JapaneseEnvelopeYouNumber4)return PageMediaSizeName.JapanYou4Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.JapaneseEnvelopeYouNumber4Rotated)return PageMediaSizeName.JapanYou4EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.JapanesePostcard)return PageMediaSizeName.JapanHagakiPostcard;
            if (pageKind == System.Drawing.Printing.PaperKind.JapanesePostcardRotated)return PageMediaSizeName.JapanHagakiPostcardRotated;
            //if (pageKind == System.Drawing.Printing.PaperKind.Ledger)return PageMediaSizeName.le;
            if (pageKind == System.Drawing.Printing.PaperKind.Legal) return PageMediaSizeName.NorthAmericaLegal;
            if (pageKind == System.Drawing.Printing.PaperKind.LegalExtra) return PageMediaSizeName.NorthAmericaLegalExtra;
            if (pageKind == System.Drawing.Printing.PaperKind.Letter) return PageMediaSizeName.NorthAmericaLetter;
            if (pageKind == System.Drawing.Printing.PaperKind.LetterExtra) return PageMediaSizeName.NorthAmericaLetterExtra;
            //if (pageKind == System.Drawing.Printing.PaperKind.LetterExtraTransverse) return PageMediaSizeName.NorthAmericaLegal;
            if (pageKind == System.Drawing.Printing.PaperKind.LetterPlus) return PageMediaSizeName.NorthAmericaLetterPlus;
            if (pageKind == System.Drawing.Printing.PaperKind.LetterRotated) return PageMediaSizeName.NorthAmericaLetterRotated;
            //if (pageKind == System.Drawing.Printing.PaperKind.LetterSmall) return PageMediaSizeName.NorthAmericaLegal;
            //if (pageKind == System.Drawing.Printing.PaperKind.LetterTransverse)return PageMediaSizeName.;
            if (pageKind == System.Drawing.Printing.PaperKind.MonarchEnvelope)return PageMediaSizeName.NorthAmericaMonarchEnvelope;
            if (pageKind == System.Drawing.Printing.PaperKind.Note)return PageMediaSizeName.NorthAmericaNote;
            if (pageKind == System.Drawing.Printing.PaperKind.Number10Envelope) return PageMediaSizeName.NorthAmericaNumber10Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.Number11Envelope) return PageMediaSizeName.NorthAmericaNumber11Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.Number12Envelope) return PageMediaSizeName.NorthAmericaNumber12Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.Number14Envelope) return PageMediaSizeName.NorthAmericaNumber14Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.Number9Envelope) return PageMediaSizeName.NorthAmericaNumber9Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.PersonalEnvelope) return PageMediaSizeName.NorthAmericaPersonalEnvelope;
            if (pageKind == System.Drawing.Printing.PaperKind.Prc16K) return PageMediaSizeName.PRC16K;
            if (pageKind == System.Drawing.Printing.PaperKind.Prc16KRotated)return PageMediaSizeName.PRC16KRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.Prc32K)return PageMediaSizeName.PRC32K;
            if (pageKind == System.Drawing.Printing.PaperKind.Prc32KBig)return PageMediaSizeName.PRC32KBig;
            //if (pageKind == System.Drawing.Printing.PaperKind.Prc32KBigRotated)return PageMediaSizeName.PRC32KRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.Prc32KRotated)return PageMediaSizeName.PRC32KRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber1)return PageMediaSizeName.PRC1Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber10)return PageMediaSizeName.PRC10Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber10Rotated)return PageMediaSizeName.PRC10EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber1Rotated)return PageMediaSizeName.PRC1EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber2)return PageMediaSizeName.PRC2Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber2Rotated)return PageMediaSizeName.PRC2EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber3)return PageMediaSizeName.PRC3Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber3Rotated)return PageMediaSizeName.PRC3EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber4)return PageMediaSizeName.PRC4Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber4Rotated)return PageMediaSizeName.PRC4EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber5)return PageMediaSizeName.PRC5Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber5Rotated)return PageMediaSizeName.PRC5EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber6)return PageMediaSizeName.PRC6Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber6Rotated)return PageMediaSizeName.PRC6EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber7)return PageMediaSizeName.PRC7Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber7Rotated)return PageMediaSizeName.PRC7EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber8)return PageMediaSizeName.PRC8Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber8Rotated)return PageMediaSizeName.PRC8EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber9)return PageMediaSizeName.PRC9Envelope;
            if (pageKind == System.Drawing.Printing.PaperKind.PrcEnvelopeNumber9Rotated)return PageMediaSizeName.PRC9EnvelopeRotated;
            if (pageKind == System.Drawing.Printing.PaperKind.Quarto) return PageMediaSizeName.NorthAmericaQuarto;
            if (pageKind == System.Drawing.Printing.PaperKind.Standard10x11)return PageMediaSizeName.NorthAmerica10x11;
            if (pageKind == System.Drawing.Printing.PaperKind.Standard10x14)return PageMediaSizeName.NorthAmerica10x14;
            if (pageKind == System.Drawing.Printing.PaperKind.Standard11x17)return PageMediaSizeName.NorthAmerica11x17;
            //if (pageKind == System.Drawing.Printing.PaperKind.Standard12x11)return PageMediaSizeName.;
            //if (pageKind == System.Drawing.Printing.PaperKind.Standard15x11)return PageMediaSizeName.;
            if (pageKind == System.Drawing.Printing.PaperKind.Standard9x11) return PageMediaSizeName.NorthAmerica9x11;
            if (pageKind == System.Drawing.Printing.PaperKind.Statement) return PageMediaSizeName.NorthAmericaStatement;
            if (pageKind == System.Drawing.Printing.PaperKind.Tabloid) return PageMediaSizeName.NorthAmericaTabloid;
            if (pageKind == System.Drawing.Printing.PaperKind.TabloidExtra) return PageMediaSizeName.NorthAmericaTabloidExtra;
            //if (pageKind == System.Drawing.Printing.PaperKind.USStandardFanfold) return PageMediaSizeName.NorthAmericaStatement;

            return PageMediaSizeName.Unknown;
        }
        #endregion

        #region GetRanges
        [System.Security.SecuritySafeCritical()]
        public static List<Range> GetRanges(StiReport report, int fromPageGlobal, int toPageGlobal)
        {
            #region Create List of Ranges
            var ranges = new List<Range>();
            int startIndex = 0;
            int fromPage = 1;
            int toPage = 1;

            var pages = new List<StiPage>();
            foreach (StiPage page in report.RenderedPages)
            {
                for (int x = 0; x < page.SegmentPerWidth; x++)
                {
                    for (int y = 0; y < page.SegmentPerHeight; y++)
                    {
                        pages.Add(page);
                    }
                }
            }
            
            foreach (var page in pages)
            {
                var startPage = report.RenderedPages[startIndex];

                if (!(startPage.Orientation == page.Orientation &&
                    startPage.PageWidth == page.PageWidth &&
                    startPage.PageHeight == page.PageHeight &&
                    startPage.PaperSize == page.PaperSize))
                {
                    ranges.Add(new Range(fromPage, toPage - 1));
                                        
                    fromPage = toPage;
                    startIndex = toPage - 1;
                }
                toPage++;
            }
            if (toPage != startIndex - 1)
            {
                ranges.Add(new Range(fromPage, toPage - 1));
            }
            #endregion

            #region Apply global range
            int rangeIndex = 0;
            while (rangeIndex < ranges.Count)
            {
                Range range = ranges[rangeIndex];

                if (range.fromPage > toPageGlobal) ranges.RemoveAt(rangeIndex);
                else if (range.toPage < fromPageGlobal) ranges.RemoveAt(rangeIndex);
                else 
                {
                    if (range.fromPage < fromPageGlobal)range.fromPage = fromPageGlobal;
                    if (range.toPage > toPageGlobal)range.toPage = toPageGlobal;
                    rangeIndex++;
                }
            }
            #endregion

            return ranges;
        }
        #endregion

        #endregion

        public StiWpfPrintProviderTest(StiReport report)
        {
            this.report = report;
        }       
    }

    public class StiWpfDocumentPaginatorTest : DocumentPaginator
    {
        #region Fields
        private StiReport report;
        private int fromPage = -1;
        private int toPage = -1;
        private double printableWidth;
        private double printableHeight;
        #endregion

        #region Properties
        public override bool IsPageCountValid
        {
            get
            {
                return true;
            }
        }

        private int pageCount;
        public override int PageCount
        {
            get
            {
                return pageCount;
            }
        }

        public override Size PageSize
        {
            get
            {
                return Size.Empty;
            }
            set
            {
            }
        }

        public override IDocumentPaginatorSource Source
        {
            get
            {
                return null;
            }
        }
        #endregion

        #region Methods
        public override DocumentPage GetPage(int pageNumber)
        {
            int x = 0;
            int y = 0;
            report.CurrentPrintPage = report.GetPageIndex(pageNumber + this.fromPage, out x, out y);

            StiPage page = report.RenderedPages[report.CurrentPrintPage];

            if (page.Report.RenderedPages.CacheMode)
                page.Report.RenderedPages.GetPage(page);

            page.CurrentWidthSegment = x;
            page.CurrentHeightSegment = y;
            page.IsPrinted = true;

            #region Create Visual
            DrawingVisual visual = new DrawingVisual();
            DrawingContext dc = visual.RenderOpen();

            double pageWidth = page.Unit.ConvertToHInches(page.DisplayRectangle.Width);
            double pageHeight = page.Unit.ConvertToHInches(page.DisplayRectangle.Height);
            double mgLeft = page.Unit.ConvertToHInches(page.Margins.Left);
            double mgTop = page.Unit.ConvertToHInches(page.Margins.Top);
            double mgBottom = page.Unit.ConvertToHInches(page.Margins.Bottom);
            double mgRight = page.Unit.ConvertToHInches(page.Margins.Right);

            int segmentPerWidth = Math.Max(page.SegmentPerWidth, 1);
            int segmentPerHeight = Math.Max(page.SegmentPerHeight, 1);

            double pgWidth = (pageWidth - mgLeft - mgRight) / segmentPerWidth + mgLeft + mgRight;
            double pgHeight = (pageHeight - mgTop - mgBottom) / segmentPerHeight + mgTop + mgBottom;

            double factorX = 0.96;
            double factorY = 0.96;

            dc.PushTransform(new ScaleTransform(factorX, factorY));

            #region Paint corner markers for correct result visual size
            Pen cornerPen = new Pen(new SolidColorBrush(Stimulsoft.Report.Wpf.StiColorHelper.ColorToWpfColor(System.Drawing.Color.Transparent)), 1);
            Stimulsoft.Report.Wpf.StiWpfDrawing.DrawLine(dc, cornerPen, 1, 1, pgWidth - 1, 1);
            Stimulsoft.Report.Wpf.StiWpfDrawing.DrawLine(dc, cornerPen, 1, pgHeight - 1, pgWidth - 1, pgHeight - 1);
            #endregion

            #region Add Transformation for Segmented Pages
            if (page.SegmentPerWidth > 1 || page.SegmentPerHeight > 1)
            {
                double pgWd = (page.PageWidth - page.Margins.Left - page.Margins.Right);
                double pgHt = (page.PageHeight - page.Margins.Top - page.Margins.Bottom);

                double scLeft = page.Unit.ConvertToHInches(page.CurrentWidthSegment * pgWd);
                double scTop = page.Unit.ConvertToHInches(page.CurrentHeightSegment * pgHt);

                dc.PushTransform(new TranslateTransform(-scLeft, -scTop));
                dc.PushClip(new RectangleGeometry(new Rect(
                    scLeft + mgLeft,
                    scTop + mgTop, page.Unit.ConvertToHInches(pgWd), page.Unit.ConvertToHInches(pgHt))));
            }
            #endregion

            #region Draw Page
            report.IsPrinting = true;
            Stimulsoft.Report.Events.StiPaintEventArgs ee = new Stimulsoft.Report.Events.StiPaintEventArgs(dc, Stimulsoft.Base.Drawing.RectangleD.Empty);

            //page.Paint(ee);
            StiPageWpfPainterTest pt = new StiPageWpfPainterTest();
            pt.Paint(page, ee);

            report.IsPrinting = false;
            #endregion

            #region Remove Transformation for Segmented Pages
            if (page.SegmentPerWidth > 1 || page.SegmentPerHeight > 1)
            {
                dc.Pop();
                dc.Pop();
            }
            #endregion

            dc.Pop();

            dc.Close();
            #endregion

            report.PageNumber++;

            return new DocumentPage(visual);
        }
        #endregion

        public StiWpfDocumentPaginatorTest(StiReport report, int fromPage, int toPage, double printableWidth, double printableHeight)
        {
            this.fromPage = fromPage;
            this.toPage = toPage;
            this.report = report;
            this.pageCount = toPage - fromPage + 1;
            this.printableWidth = printableWidth;
            this.printableHeight = printableHeight;
        }
    }

}
